****************************************************************************
*                    Multi-line TBrowse example                            *
*                                                                          *
*  This will help to demonstrate the flexability of TBrowse.  Specifically *
*  it browses a customer.dbf in the traditional format of 1 line per       *
*  record, but pressing CTRL-T toggles the display to a multiple line per  *
*  record format.  Note that some of the records contain 3 address lines   *
*  and others have 4.  I appologize for the lack of clear documentation,   *
*  but I have been very short on time lately.  Have Fun, I would love to   *
*  see what else you think of when exploring the posibilities of TBrowse.  *
*  To compile:                                                             *
*              CLIPPER tb_multi                                            *
*              RTLINK FI tb_multi /PLL:BASE50                              *
*                                                                          *
*     - Troy Henikoff   (CIS - 76436,411)                                  *
****************************************************************************


# include "inkey.ch"

# define K_CURRENT_LINE     1
# define K_SINGLE_LINE      2
LOCAL t_browse, t_column, t_cargo_array[2]

USE Customer
INDEX ON Customer TO Temp.bak
GOTO TOP

t_browse           := TBrowseDB(5,1,20,78)
t_browse:SkipBlock := {|x| Cust_Skipper(x, t_browse)}

t_column := TBColumnNew( "", {|| disp_cust_line(t_browse)} )  && The one column
t_browse:AddColumn(t_column)

t_cargo_array[K_SINGLE_LINE]  := .T.   && BOOLEAN .T. -> Only display one line per record, .F. -> Multiple lines per record
t_cargo_array[K_CURRENT_LINE] := 1     && The current line of the current customer = 1
t_browse:Cargo := t_cargo_array        && Store the array in cargo
CLEAR
@1,28 SAY "MULTI-LINE TBROWSE DEMO"
@23,5 SAY '"ALT-T":Toggle between modes   "ALT-M":Memory report   "ESC":Exit'
activate_browse(t_browse)              && Now that we have built the browse object, make it do something!

CLEAR
RETURN


***********
* Function: activate_browse
* Author..: Troy Henikoff
* Date....: 09/22/90
* Notes...: "Activates" a tBrowse object
*         : t_request follows the same conventions that the return values
*         : from a UDF in dbedit() did.
*
* Params..: p_browse - REFERENCE to which object to activate
*
* Returns.: NIL
***********

FUNCTION activate_browse(p_browse)
  LOCAL t_request:=3, t_key, t_screen, t_dummy:={}, t_recno

  DO WHILE .NOT. 0==t_request
    t_key := 0                                   && Reset value of t_key
    IF t_request==2                              && IF we have to re-draw entire screen
      p_browse:RefreshAll()                      &&   Set all lines in the browse to be refreshed
    ENDIF
    t_request := 3                               && SET t_request to "Continue"

    *** stabilize the display (standard TBrowse stuff) ***
    DO WHILE .NOT. p_browse:stabilize()
      t_key := INKEY()
      IF ( t_key != 0 )
        EXIT 			                               && abort IF a key is waiting
      ENDIF
    ENDDO


    IF ( p_browse:hitTop .or. p_browse:hitBottom )
      CLEAR TYPEAHEAD
    ENDIF

    *** This part is needed for multiuser applications to ensure that we are displaying the most current version of the data ***
    IF t_key == 0                                && If the browse was completely re-drawn,
      p_browse:RefreshCurrent()                  &&   set the "Current" line as HOT
      DO WHILE .NOT. p_browse:stabilize()        &&   re-draw (hopefully just 1 line)
      ENDDO
    ENDIF
    SETCURSOR(0)                                 && Set cursor off while waiting

    IF ( p_browse:stable )                       && display is stable
      t_key := INKEY(0)
    ENDIF


    *** process key ***
    DO CASE
      CASE ( t_key == K_DOWN )
        p_browse:down()

      CASE ( t_key == K_UP )
        p_browse:up()

      CASE ( t_key == K_PGDN )
        p_browse:pageDown()

      CASE ( t_key == K_PGUP )
        p_browse:pageUp()

      CASE ( t_key == K_ESC )
        t_request := 0

      *** "T"oggle between single and multiple line display ***
      CASE t_key==K_ALT_T
        p_browse:Cargo[K_SINGLE_LINE] := .NOT. p_browse:Cargo[K_SINGLE_LINE]
        t_request = 2

      *** "M"emory status display ***
      CASE ( t_key == K_ALT_M )
        t_screen := SAVESCREEN(12,20,18,55)
        @ 12,20 CLEAR TO 18,55
        @ 12,20,18,55 BOX "͸Գ"
        TONE(500,5)
        @13,23 SAY "Memory(0) - Characters....:"+LTRIM(STR(Memory(0)))+"K"
        @14,23 SAY "Memory(1) - Largest Block.:"+LTRIM(STR(Memory(0)))+"K"
        @15,23 SAY "Memory(2) - Run Commands..:"+LTRIM(STR(Memory(0)))+"K"

        @17,25 SAY "Press any key to continue..."
        
        INKEY(0)
        RESTSCREEN(12,20,18,55,t_screen)
        t_request := 3

      OTHERWISE
        TONE(200,10)

    ENDCASE
  ENDDO
RETURN(NIL)


***********
* Function: Cust_skipper()
* Author..: Troy Henikoff
* Date....: 09/22/90
* Notes...: Custom Skipper
*
* Params..: p_num_to_skip - NUMBER of records to skip
*           p_browse      - REFERENCE to browse that we are object skipping in
*
* Returns.: NUMBER of records skipped
***********

FUNCTION cust_skipper(p_num_to_skip, p_browse)

  LOCAL t_num_skipped                  && the number of screen lines actually skipped
  t_num_skipped   := 0                 && none skipped yet...
  DO CASE
    CASE p_num_to_skip == 0                                   && Special case
      SKIP 0

    CASE (p_browse:HitBottom .AND. p_num_to_skip>0) .OR.;     && "Short circuit" if we already heve hit the top/bottom
         (p_browse:HitTop    .AND. p_num_to_skip<0)
      SKIP 0

    CASE ( p_num_to_skip > 0 .and. RECNO() != LASTREC() + 1 ) && Not at phantom record yet
      p_browse:HitTop := .F.                                  && Since we have been asked to skip down, we are no longer sitting at the top.
      DO WHILE ( t_num_skipped < p_num_to_skip )              && Try to skip as many lines as requested
        IF p_browse:Cargo[K_CURRENT_LINE] < num_lines(p_browse:Cargo[K_SINGLE_LINE])       && IF the current line for this customer < the total number of lines for this customer
          p_browse:Cargo[K_CURRENT_LINE]++                    &&   then just flag that we are on the next line
        ELSE                                                  && ELSE  
          SKIP 1                                              &&   Skip to next record
          IF EOF()                                            &&   IF EOF()
            p_browse : hitBottom := .T.                       &&     Flag hitbottom
            SKIP -1                                           &&     Skip back to last record
            EXIT                                              &&     EXIT the loop
          ELSE                                                &&   ELSE
            p_browse:Cargo[K_CURRENT_LINE] := 1               &&     We are on the first line of the next customer
          ENDIF                                               &&   ENDIF
        ENDIF                                                 && ENDIF
        t_num_skipped++                                       && Increment the number of lines that have been skipped
      ENDDO

    OTHERWISE                                                 && ( p_num_to_skip < 0 )  Same as above, but skipping backwards.
      p_browse:HitBottom := .F.
      DO WHILE ( t_num_skipped > p_num_to_skip )
        IF p_browse:Cargo[K_CURRENT_LINE] > 1
          p_browse:Cargo[K_CURRENT_LINE]--
        ELSE
          SKIP -1
          IF BOF()
            p_browse : hitTop := .T.
            EXIT
          ELSE
            p_browse:Cargo[K_CURRENT_LINE] := num_lines(p_browse:Cargo[K_SINGLE_LINE])
          ENDIF
        ENDIF

        t_num_skipped--
      ENDDO
  ENDCASE
RETURN(t_num_skipped)

***********
* Function: disp_cust_line()
* Author..: Troy Henikoff
* Date....: 12/27/90
* Notes...: Display one line of customer data
*
* Params..: p_browse - REFERENCE to active browse object
*
* Returns.: STRING TO DISPLAY
***********

FUNCTION disp_cust_line(p_browse)
  DO CASE
    CASE p_browse:Cargo[K_CURRENT_LINE]==1               && If the "current" line is 1
      t_return = SUBSTR(Customer+" : "+Name,1,30)        &&   display first address line

    CASE p_browse:Cargo[K_CURRENT_LINE]==2               && If the "current" line is 2
      t_return = SPACE(9)+Addr_1                         &&   display second address line

    CASE p_browse:Cargo[K_CURRENT_LINE]==3 .AND. .NOT. EMPTY(Addr_2)  && If it is appropriate to display the extra address line
      t_return = SPACE(9)+Addr_2                                      &&   Display it

    OTHERWISE                                            && Otherwise it is time for city, state, zip....
      t_return = SPACE(9)+SUBSTR(TRIM(City)+", "+State+" "+Zip+SPACE(30),1,30)
  ENDCASE
RETURN(t_return)


***********
* Function: num_lines()
* Author..: Troy Henikoff
* Date....: 12/27/90
* Notes...: Calculates to be displayed the number of lines in a customer record
*
* Params..: p_single - BOOLEAN .T. if one line mode, .F. if multiline mode
*
* Returns.: NUMBER of lines
***********

FUNCTION num_lines(p_single)
  LOCAL t_return
  IF p_single
    t_return := 1
  ELSE
    t_return := IF(EMPTY(Addr_2),3,4)
  ENDIF
RETURN(t_return)

